<?php
################################################################################
### This disclaimer must be kept intact in order to use this product.        ###
################################################################################
### Project: jT CarFramework [http://intersofts.com]
### Author: J. Toshio Taniguchi
### Since: 27.04.2011
### Version: 1.0.0
### Copyrights: J. Toshio Taniguchi
### Contact: j.taniguchi@taniguchi-blog.com
################################################################################
### CONTRIBUTORS
### - none
################################################################################

class adminthememanager {
    protected $lang;
    protected $form_status;
    protected $form_message;
    protected $form_style;
    private $theme_dir;
    private $theme_list;

    /**
     * Constructor
     * <br>---------------------------------------------------------------------
     */
    public function adminthememanager($lang=array()){
        $this->lang = $lang;
        
        // handle remove theme request.
        if( isset($_POST['removebtn']) ){
            $this->handleRemoveThemeForm();
        }
        
        // handle apply theme request.
        if( isset($_POST['applybtn']) ){
            $this->handleApplyThemeForm();
        }
        
        // handle install new theme package.
        if( isset($_POST['savebtn']) ){
            $this->handleSaveThemeForm();
        }
        
        // load theme directory.
        $this->loadThemeDir();
        
        // load installed theme list.
        $this->loadInstalledThemeList();
    }
    
    /**
     * Private method: handle save theme package
     * <br>---------------------------------------------------------------------
     * @return void
     */
    private function handleSaveThemeForm(){
        $extension = $_FILES['fileInput']['name'];
        $filesize  = $_FILES['fileInput']['size'];
        $filename  = $extension;
        
        // validate uploaded file.
        if( !isset($_FILES['fileInput']) || $filesize < 1 || empty($filename) ){
            $this->form_message = $this->lang['THEME_MANAGER_INVALID_FILE_ERROR'];
            $this->form_status = true;
            $this->form_style = 'alert-error';
            return;
        }
        
        $extension = explode('.',$extension);
        $extension = $extension[(count($extension)-1)];
        $extension = strtolower($extension);
        
        if( $extension != 'zip' ){
            $this->form_message = $this->lang['THEME_MANAGER_INVALID_FILE_ERROR'];
            $this->form_status = true;
            $this->form_style = 'alert-error';
            return;
        }
        
        // filter package name.
        require_once BASE_CLASS . 'class-utilities.php';
        
        $filename = str_replace('.zip', '', $filename);
        $filename = str_replace('.ZIP', '', $filename);
        $filename = Utilities::genetateSlug($filename);
        
        // check if filename still good after sanitize.
        if( empty($filename) ){
            $this->form_message = $this->lang['THEME_MANAGER_INVALID_PACKAGE_ERROR'];
            $this->form_status = true;
            $this->form_style = 'alert-error';
            return;
        }
        
        // check if the theme is already installed.
        $path = BASE_ROOT . 'view/theme/' . $filename . '/';
        
        if( file_exists($path) ){
            $this->form_message = $this->lang['THEME_MANAGER_THEME_INSTALLED_ERROR'];
            $this->form_status = true;
            $this->form_style = 'alert-error';
            return;
        }
        
        // save uploaded file to root directory.
        $destination = BASE_ROOT . 'tmp/' . $filename.'.zip';
        
        if( !@move_uploaded_file($_FILES['fileInput']['tmp_name'],$destination) ){
            require_once BASE_CLASS . 'class-log.php';
            
            LogReport::write('Unable to upload theme package. Theme package size is too large. Increase the maximum uploaded file size on your PHP.INI.');
            
            $this->form_message = $this->lang['THEME_MANAGER_UPLOAD_ERROR'];
            $this->form_status = true;
            $this->form_style = 'alert-error';
            return;
        }
        
        // create the theme directory in which files will be extracted.
        if( !@mkdir($path,0777) ){
            require_once BASE_CLASS . 'class-log.php';
            
            LogReport::write('Unable create theme directory at ' . __FILE__ . ':' . __LINE__ . '. Check file permission.');
            
            $this->form_message = $this->lang['THEME_MANAGER_UNZIP_ERROR'];
            $this->form_status = true;
            $this->form_style = 'alert-error';
            
            @unlink($destination);
            
            return;
        }
        
        // unpack the package to the theme directory.
        $zip = new ZipArchive();
        
        if( $zip->open($destination) === TRUE ){
            $zip->extractTo($path);
            $zip->close();
        } else {
            // something went wrong...
            require_once BASE_CLASS . 'class-log.php';
            
            LogReport::write('Unable to unzip uploaded theme package at ' . __FILE__ . ':' . __LINE__ . '. Check if GZIP is enabled in your server.');
            
            $this->form_message = $this->lang['THEME_MANAGER_UNZIP_ERROR'];
            $this->form_status = true;
            $this->form_style = 'alert-warning';
            
            @unlink($destination);
            
            return;
        }
        
        // remove theme zip package.
        @unlink($destination);
        
        $this->form_message = $this->lang['THEME_MANAGER_THEME_INSTALL_SUCCESS'];
        $this->form_status = true;
        $this->form_style = 'alert-success';
        return;
    }
    
    /**
     * Private method: apply theme form handler
     * <br>---------------------------------------------------------------------
     * @return void
     */
    private function handleApplyThemeForm(){
        $theme = trim($_POST['themeTitleHid']);
        
        // make sure the directory exists.
        if( empty($theme) ){
            return;
        }
        
        $path = BASE_ROOT . 'view/theme/' . $theme . '/';
        
        if( !file_exists($path) ){
            return;
        }
        
        // set default theme on table setting.
        require_once BASE_CLASS . 'class-connect.php';
        
        $cnx = new Connect();
        $cnx->open();
        
        $theme = @mysql_real_escape_string($theme);
        
        if( !@mysql_query("UPDATE `setting` SET `default_theme`='$theme'") ){
            $cnx->close();
            
            require_once BASE_CLASS . 'class-log.php';
            
            LogReport::write('Unable to activate theme due a query error at ' . __FILE__ . ':' . __LINE__);
            
            $this->form_message = $this->lang['THEME_MANAGER_ACTIVE_THEME_ERROR'];
            $this->form_status = true;
            $this->form_style = 'alert-warning';
            return;
        }
        
        $cnx->close();
        $this->form_message = $this->lang['THEME_MANAGER_ACTIVE_THEME_SUCCESS'];
        $this->form_status = true;
        $this->form_style = 'alert-success';
        return;
    }
    
    /**
     * Private method: remove theme form handler
     * <br>---------------------------------------------------------------------
     * @return void
     */
    private function handleRemoveThemeForm(){
        $theme = trim($_POST['themeTitleHid']);
        
        // make sure the directory exists and it's not the default.
        if( empty($theme) || $theme == 'jtheme' ){
            return;
        }
        
        $path = BASE_ROOT . 'view/theme/' . $theme . '/';
        
        if( !file_exists($path) ){
            return;
        }
        
        require_once BASE_CLASS . 'class-utilities.php';
        
        if( !Utilities::remove_directory($path) ){
            require_once BASE_CLASS . 'class-log.php';
            
            LogReport::write('Unable to remove theme directory at ' . $path . ' due an error throw at ' . __FILE__ . ':' . __LINE__);
            $this->form_message = $this->lang['THEME_MANAGER_REMOVE_THEME_ERROR'];
            $this->form_status = true;
            $this->form_style = 'alert-warning';
            return;
        }
        
        $this->form_message = $this->lang['THEME_MANAGER_REMOVE_THEME_SUCCESS'];
        $this->form_status = true;
        $this->form_style = 'alert-success';
        return;
    }
    
    /**
     * Private method: load installed theme list
     * <br>---------------------------------------------------------------------
     * @return array
     */
    private function loadInstalledThemeList(){
        $theme_path = BASE_ROOT . 'view/theme/';
        $error = false;
        
        if( !$dir = @opendir($theme_path) ){
            require_once BASE_CLASS . 'class-log.php';
            LogReport::write('Unable to load theme list. opendir() error at ' . __FILE__ . ':' . __LINE__);
            $this->form_message = $this->lang['THEME_MANAGER_LIST_LOADER_ERROR'];
            $this->form_status = true;
            $this->form_style = 'alert-warning';
            return;
        }
        
        $this->theme_list = array();
        
        while( false !== ($file = @readdir($dir)) ){
            if( $file != '.htaccess' && $file != 'index.php' && $file != '..' && $file != '.' ){
                // make sure the theme's setting.php file exists.
                if( file_exists(BASE_ROOT . 'view/theme/' . $file . '/setting.php') ){
                    array_push($this->theme_list,$file);
                }
            }
        }
        
        @closedir($dir);
        
        return;
    }
    
    /**
     * Private method: get theme name
     * <br>---------------------------------------------------------------------
     * @return void
     */
    private function loadThemeDir(){
        require_once BASE_CLASS . 'class-connect.php';
        
        $cnx = new Connect();
        $cnx->open();
        
        if( !$sql = @mysql_query("SELECT `default_theme` FROM `setting` LIMIT 1;") ){
            require_once BASE_CLASS . 'class-log.php';
            
            LogReport::write('Unable to load default theme name due a query error at ' . __FILE__ . ':' . __LINE__);
            $this->form_message = $this->lang['THEME_MANAGER_LOADER_QUERY_ERROR'];
            $this->form_style = 'alert-warning';
            $this->form_status = true;
            $cnx->close();
        }
        
        $r = @mysql_fetch_assoc($sql);
        @mysql_free_result($sql);
        $cnx->close();
        
        $this->theme_dir = trim($r['default_theme']);
        
        return;
    }
    
    /**
     * Public method: get installed theme list
     * <br>---------------------------------------------------------------------
     * @return array
     */
    public function getInstalledThemeList(){
        return $this->theme_list;
    }
    
    /**
     * Public method: get theme directory name
     * <br>---------------------------------------------------------------------
     * @return string
     */
    public function getThemeDir(){
        return $this->theme_dir;
    }
    
    /**
     * Public method: get form status
     * <br>---------------------------------------------------------------------
     * @return bool.
     */
    public function getFormStatus()
    {
        return $this->form_status;
    }

    /**
     * Public method: get form message 
     * <br>---------------------------------------------------------------------
     * @return string.
     */
    public function getFormMessage()
    {
        return $this->form_message;
    }

    /**
     * Public method: get form style
     * <br>---------------------------------------------------------------------
     * @return string.
     */
    public function getFormStyle()
    {
        return $this->form_style;
    }
}